/*********************************************************************
  Kitt Widget

*********************************************************************/

#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <math.h>

#include "KittP.h"

/*********************************************************************
  Function prototypes

*********************************************************************/

/* Actions */

/* Core Class methods */
static void				KittInitialize(Widget, Widget, ArgList, Cardinal*);
static void				KittRealize(Widget, XtValueMask*, XSetWindowAttributes*);
static void				KittResize(Widget);
static void				KittDestroy(Widget);
static void				KittExpose(Widget, XEvent*, Region);
static Boolean			KittSetValues(Widget, Widget, Widget, ArgList, Cardinal*);
static XtGeometryResult	KittQueryGeometry(Widget, XtWidgetGeometry*, XtWidgetGeometry*);

/* Misc */
static void		TimerCallbackProc(XtPointer, XtIntervalId*);

/*********************************************************************
  Translations

*********************************************************************/

/*********************************************************************
  Actions

*********************************************************************/

/*********************************************************************
  Defines and macros

*********************************************************************/

#define WIDGET(w)  ((KittWidget)(w))
#define CORE(a)    (WIDGET(a)->core)
#define KITT(a)    (WIDGET(a)->kitt)

#define New(t, n)  ((t*) XtMalloc(n * sizeof(t)))
#define Free(p)		XtFree((char*)p)

/*********************************************************************
  Widget Resources

*********************************************************************/

#define TheOffset(field) XtOffset(KittWidget, kitt.field)

static XtResource resources[] = 
{
	{
		XtNinterval,
		XtCInterval,
		XtRInt,
		sizeof(unsigned int),
		TheOffset(interval),
		XtRImmediate,
		(XtPointer) 100
	},
	{
		XtNsegments,
		XtCSegments,
		XtRInt,
		sizeof(int),
		TheOffset(num_segments),
		XtRImmediate,
		(XtPointer) 7
	}
};

#undef TheOffset

/*********************************************************************
  THE Kitt Class Record

*********************************************************************/

KittClassRec kittClassRec = 
{
	/*
	 * CoreClassPart
	 */
	{
		(WidgetClass)&widgetClassRec,			/* superclass            */
		"Kitt",									/* class_name            */
		sizeof(KittRec),						/* widget_size           */
		NULL,									/* class_initialize      */
		NULL,									/* class_part_initialize */
		FALSE,									/* class_init            */
		KittInitialize,							/* initialize            */
		NULL,									/* initialize_hook       */
		KittRealize,							/* realize               */
		NULL,									/* actions               */
		0,										/* num_actions           */
		resources,								/* resources             */
		XtNumber(resources),					/* num_resources         */
		NULLQUARK,								/* xrm_class             */
		TRUE,									/* compress_motion       */
		XtExposeCompressMultiple |
		XtExposeGraphicsExposeMerged,			/* compress_exposure     */
		TRUE,									/* compress_enterleave   */
		TRUE,									/* visible_intress       */
		KittDestroy,							/* destroy               */
		KittResize,								/* resize                */
		KittExpose,								/* expose                */
		KittSetValues,							/* set_values            */
		NULL,									/* set_values_hook       */
		XtInheritSetValuesAlmost,				/* set_values_almost     */
		NULL,									/* get_values_hook       */
		XtInheritAcceptFocus,					/* accept_focus          */
		XtVersion,								/* version               */
		NULL,									/* callback_private      */
		XtInheritTranslations,					/* tm_translations       */
		KittQueryGeometry,						/* query_geometry        */
		XtInheritDisplayAccelerator,			/* display accel         */
		NULL,									/* extension             */
	},

	/*
	 * KittClassPart
	 */
	{ 
		0,										/* empty */
	}
};

/* This WidgetClass data structure is publicly extern'ed in Kitt.h */

WidgetClass kittWidgetClass = (WidgetClass) &kittClassRec;

/*********************************************************************
  Kitt Class methods

/********************************************************************/

/*********************************************************************

  KittInitialize()

      The main widget instance initialization routine.  Initialize the
	  instance variables here, and verify the resources values.

*********************************************************************/

static void KittInitialize(Widget		request_widget,
						   Widget 		new_widget,
						   ArgList		args,
						   Cardinal*	num_args)
{
	int			i;
	XColor		fgcolor;

	KITT(new_widget).decay_rate		= 25;
	KITT(new_widget).num_colors		= 4; /* must be at leat two */

	KITT(new_widget).blip			= 0;
	KITT(new_widget).blip_width		= 0;
	KITT(new_widget).blip_height	= 0;
	KITT(new_widget).direction		= Right;
	KITT(new_widget).interval_id	= 0;

	KITT(new_widget).lights	= New(Light, KITT(new_widget).num_segments);
	for (i = 0; i < KITT(new_widget).num_segments; i++)
	{
		KITT(new_widget).lights[i].intensity = 0;
	}
	KITT(new_widget).lights[KITT(new_widget).blip].intensity = 100;

	fgcolor.red   = 0xFFFF;
	fgcolor.green = 0x0;
	fgcolor.blue  = 0x0;

	KITT(new_widget).colors = New(Pixel, KITT(new_widget).num_colors);
	for (i = 0; i < KITT(new_widget).num_colors; i++)
	{
		Status	result;
		XColor	xcolor;

		xcolor.flags = DoRed | DoGreen | DoBlue;
		xcolor.red	 = i * (fgcolor.red   / (KITT(new_widget).num_colors - 1));
		xcolor.green = i * (fgcolor.green / (KITT(new_widget).num_colors - 1));
		xcolor.blue	 = i * (fgcolor.blue  / (KITT(new_widget).num_colors - 1));

		result = XAllocColor(XtDisplay(new_widget),
							 CORE(new_widget).colormap,
							 &xcolor);

		if (result == 0)
		{
			XtError("Failed to allocate colors");
			break;
		}

		KITT(new_widget).colors[i] = xcolor.pixel;
	}

 	CORE(new_widget).background_pixel = BlackPixel(XtDisplay(new_widget),
												   DefaultScreen(XtDisplay(new_widget)));

	KittResize(new_widget);
}

/*********************************************************************

  KittRealize()

      Create the window associated with the widget.

*********************************************************************/

static void KittRealize(Widget					w,
						XtValueMask*			value_mask,
						XSetWindowAttributes*	window_attributes)
{
	/* Call the Realize procedure (XtInheritRealize) */
	(*kittWidgetClass->core_class.superclass->core_class.realize)(w, value_mask, window_attributes);

	/* Now we can create the GC */
	KITT(w).gc = XCreateGC(XtDisplay(w), XtWindow(w), 0, NULL);

	/* Start up the timer */
	KITT(w).interval_id = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
										  KITT(w).interval,
										  TimerCallbackProc,
										  w);
}

/*********************************************************************

  KittResize()

      Redraw according to size changes.

*********************************************************************/

static void KittResize(Widget w)
{
	KITT(w).blip_width	= CORE(w).width / KITT(w).num_segments;
	KITT(w).blip_height	= CORE(w).height;
}

/*********************************************************************

  KittDestroy()

      Clean up allocated resources when the widget is destroyed.
	  Anything that is malloc'd by the widget should be freed here.
 
*********************************************************************/

static void KittDestroy(Widget w)
{
	int i;

	if (KITT(w).interval_id != 0)
		XtRemoveTimeOut(KITT(w).interval_id);

	Free(KITT(w).lights);
	XFreeColors(XtDisplay(w),
				CORE(w).colormap,
				KITT(w).colors,
				KITT(w).num_colors,
				0);
	Free(KITT(w).colors);
	XFreeGC(XtDisplay(w),  KITT(w).gc);
}

/*********************************************************************

  KittExpose()

      Redraw region that gets exposed.
 
*********************************************************************/

static void KittExpose(Widget	w, 
					   XEvent*	event,
					   Region	region)
{
	int	i;

	if (CORE(w).visible)
	{
		unsigned int	xpos = 0;
		for (i = 0; i < KITT(w).num_segments; i++)
		{
			int		color;

			/* pick color based on intensity of light segment */
			color = KITT(w).lights[i].intensity * (KITT(w).num_colors - 1) / 100;
			
			XSetForeground(XtDisplay(w), KITT(w).gc, KITT(w).colors[color]);

			XFillRectangle(XtDisplay(w), XtWindow(w), KITT(w).gc,
						   CORE(w).x + xpos,
						   CORE(w).y,
						   KITT(w).blip_width,
						   CORE(w).height);

			xpos += KITT(w).blip_width;
  		}

	}

	/* call parent method */
	if (kittWidgetClass->core_class.superclass->core_class.expose != NULL)
		(*kittWidgetClass->core_class.superclass->core_class.expose)(w, event, region);
}

/*********************************************************************

  KittSetValues()

      The procedure is used to verify changes to the resource values.
	  If the resource changes effect the visuals of the widget, the
	  return value of the procedure should be set to True indicating
	  that the expose procedure will be called to redraw the widget.
	  Any changes to the widget's geometry in this procedure will
	  generate a geometry request to it's parent.
 
*********************************************************************/

static Boolean KittSetValues(Widget		current_widget,
							 Widget		request_widget,
							 Widget		new_widget,
							 ArgList	args,
							 Cardinal*	num_args)
{
	Boolean redisplay = False;
	Boolean resize = False;

	/* disallow background setting */
	if (CORE(new_widget).background_pixel != CORE(current_widget).background_pixel)
	{
		CORE(new_widget).background_pixel = CORE(current_widget).background_pixel;
	}

	if (KITT(new_widget).interval != KITT(current_widget).interval)
	{
		/* do nothing */
	}

	if (KITT(new_widget).num_segments != KITT(current_widget).num_segments)
	{
		int i;

		Free(KITT(new_widget).lights);

		KITT(new_widget).blip = 0;

		KITT(new_widget).lights		= New(Light, KITT(new_widget).num_segments);
		for (i = 0; i < KITT(new_widget).num_segments; i++)
		{
			KITT(new_widget).lights[i].intensity = 0;
		}
		KITT(new_widget).lights[KITT(new_widget).blip].intensity = 100;

		redisplay	= True;
		resize		= True;
	}

	if (resize == True)
		KittResize(new_widget);

	return redisplay;
}

/*********************************************************************
 
  KittQueryGeometry()

      When manager widget have a query geometry procedure it should
	  make query geometry requests to their children to calculate its
	  preferred geometry.  Once this is done, set the reply fields
	  and determine the appropriate XtGeometryResult to return.
	  
**********************************************************************/

static XtGeometryResult KittQueryGeometry(Widget			w,
										  XtWidgetGeometry*	request,
										  XtWidgetGeometry*	reply)
{
	printf("QueryGeometry\n");

	/* Set up the reply to the query */
	reply->width  = 140;
	reply->height = 10;
	reply->request_mode = CWWidth | CWHeight;
   
	/*
	 * XtGeometryYes indicates that the requested geometry is the same
	 * as reply geometry.
	 */
	if ((request->request_mode & CWWidth  && request->width  == reply->width)
	&&  (request->request_mode & CWHeight && request->height == reply->height))
		return XtGeometryYes;

	/*
	 * XtGeometryNo indicates that our current geometry is the same
	 * as reply geometry.
	 */
	if (reply->width  == CORE(w).width
	&&  reply->height == CORE(w).height)
		return XtGeometryNo;

	/*
	 * XtGeometryAlmost indicates that at least one of the fields in the 
	 * reply geometry differs from the request geometry.
	 */
	return XtGeometryAlmost;
}

/*********************************************************************
  Actions


/********************************************************************/

/*********************************************************************
  Misc

**********************************************************************/

static void TimerCallbackProc(XtPointer		data,
							  XtIntervalId* id)
{
	Widget			w = (Widget)data;
	XExposeEvent	event;
	int				i;

	/* let light intesity decay */
	/* note: should use exponential decay i think */
	for (i = 0; i < KITT(w).num_segments; i++)
	{
		if (KITT(w).lights[i].intensity - KITT(w).decay_rate < 0)
			KITT(w).lights[i].intensity = 0;
		else
			KITT(w).lights[i].intensity -= KITT(w).decay_rate;
	}

	/* if blip is at an end, change direction */
	if (KITT(w).direction == Left
	&&  KITT(w).blip      == 0)
		KITT(w).direction = Right;
	else
	if (KITT(w).direction == Right
	&&  KITT(w).blip      == KITT(w).num_segments - 1)
		KITT(w).direction = Left;

	/* move blip */
	if (KITT(w).direction == Right)
		KITT(w).blip++;
	else
		KITT(w).blip--;

	/* turn blip all the way on */
	KITT(w).lights[KITT(w).blip].intensity = 100;

	/* draw it */
	event.x		 = CORE(w).x;
	event.y		 = CORE(w).y;
	event.width	 = CORE(w).width;
	event.height = CORE(w).height;

	KittExpose(w, (XEvent*)&event, NULL);

	/* set a new timer */
	KITT(w).interval_id = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
										  KITT(w).interval,
										  TimerCallbackProc,
										  w);
}

/*********************************************************************
 External Routines
*/


/********************************************************************/
