#ifndef __KITTP_H__
#define __KITTP_H__

/*********************************************************************
 Kitt Widget
*/

#include "Kitt.h"

#include <X11/IntrinsicP.h>

#ifdef __cplusplus
extern "C" {
#endif

/*********************************************************************
  Internal types
*/

struct _Light
{
	unsigned short	intensity;		/* current intensity of light */
};
typedef struct _Light Light;

typedef enum { Left, Right } Direction;

/*********************************************************************
  Kitt part
*/

struct _KittPart
{
/* public: */
	unsigned long	interval;		/* time between blip movements */
	unsigned int	num_segments;	/* number of light segments in kitt */
	unsigned short	decay_rate;		/* % to decay after each interval */

/* private: */
	Light*			lights;			/* the array of lights */
	Pixel*			colors;			/* colors */
	unsigned int	num_colors;		/* number of colors */
	unsigned int	blip;			/* position of blip [0, segments-1] */
	Direction		direction;		/* current direction of blip */
	Dimension		blip_height;	/* current height of blip */
	Dimension		blip_width;		/* current width of blip */
	XtIntervalId	interval_id;	/* timer id */
	GC				gc;				/* graphics context for drawing */
};
typedef struct _KittPart	KittPart;

/*********************************************************************
  Kitt Record

      This is the instance structure for each instantiated Widget.
*/

struct _KittRec
{
	CorePart		core;
	KittPart		kitt;
};
typedef struct _KittRec		KittRec;

/*********************************************************************
  Kitt Class Part

     Class data members of Widget.
*/

struct _KittClassPart
{
	int		empty;
};
typedef struct _KittClassPart  KittClassPart;

/*********************************************************************
  Kitt Class Record
*/

struct _KittClassRec
{
	CoreClassPart			core_class;	
	KittClassPart			kitt_class;
};
typedef struct _KittClassRec	KittClassRec;

/*********************************************************************
  Kitt Class external definition.
*/
extern KittClassRec		kittClassRec;

/********************************************************************/

#ifdef __cplusplus
}	/* extern "C" */
#endif

/********************************************************************/
#endif /* _KITTP_H__ */
