#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xmu/Editres.h>		/* EditRes support */

#include "Kitt.h"
#include "xkitt_icon.xbm"

/*********************************************************************
  Function prototypes

*********************************************************************/

/* Actions */
static void WMProtocolAction(Widget, XEvent*, String*, Cardinal*);
static void QuitAction(Widget, XEvent*, String*, Cardinal*);

/*********************************************************************
  Actions record

*********************************************************************/

static XtActionsRec app_actions[] =
{
    { "quit",			QuitAction		 	},
    { "wm_protocol",	WMProtocolAction	}
};

/*********************************************************************
  Fallback resources for the application

*********************************************************************/

static String app_resources[] = 
{
	"XKitt.baseTranslations: #override			\\n	\
		<Message>WM_PROTOCOLS:	wm_protocol()	\\n	",
	"XKitt.accelerators: #override				\\n	\
		<Key>q:					quit()			\\n	",
	"XKitt.width:				140",
	"XKitt.height:				10",
	NULL
};

/*********************************************************************
  Command line options
  Allow the user set some Widget resources via the command line.

*********************************************************************/

static XrmOptionDescRec	app_options[] = 
{
	{"-interval",	"*Kitt.interval",	XrmoptionSepArg,	NULL},
	{"-segments",	"*Kitt.segments",	XrmoptionSepArg,	NULL},
};

static Atom XA_WM_DELETE_WINDOW;

int main(int argc, char* argv[])
{
	XtAppContext 	app;
	Widget			top_level;
	Widget			kitt;
	Pixmap			icon_pixmap;

	top_level = XtVaAppInitialize(&app,
								  "XKitt",
								  app_options,
								  XtNumber(app_options),
								  &argc, argv,
								  app_resources,
								  NULL);

	XtAppAddActions(app, app_actions, XtNumber(app_actions));

	/* support EditRes protocol */
	XtAddEventHandler(top_level, (EventMask) 0, True, _XEditResCheckMessages, NULL);

	/* set the icon of this app */
	icon_pixmap = XCreateBitmapFromData(XtDisplay(top_level),
										RootWindowOfScreen(XtScreen(top_level)),
										xkitt_icon_bitmap_bits,
										xkitt_icon_bitmap_width,
										xkitt_icon_bitmap_height);
	XtVaSetValues(top_level, XtNiconPixmap, icon_pixmap, NULL);

	/* create the KITT widget */
	kitt = XtVaCreateManagedWidget("TheKitt",
								   kittWidgetClass,
								   top_level,
								   NULL);
	XtInstallAccelerators(kitt, top_level);

	XtRealizeWidget(top_level);

	/* tell the window manager that we want to know about deleted windows */
	XA_WM_DELETE_WINDOW = XInternAtom(XtDisplay(top_level), "WM_DELETE_WINDOW", False);
	XSetWMProtocols(XtDisplay(top_level), XtWindow(top_level), &XA_WM_DELETE_WINDOW, 1);

	XtAppMainLoop(app);
}

/*********************************************************************
  Actions

*********************************************************************/

static void WMProtocolAction(Widget		w,
							 XEvent*	event,
							 String*	params,
							 Cardinal*	num_params)
{
	if (event->type == ClientMessage)
	{
		XClientMessageEvent*	cm_event = (XClientMessageEvent*) event;
		Atom					protocol = (Atom) cm_event->data.l[0];

		if (protocol == XA_WM_DELETE_WINDOW)
		{
			XtCallActionProc(w, "quit", event, NULL, 0);
		}
	}
}

static void QuitAction(Widget		w,
					   XEvent*		event,
					   String*		params,
					   Cardinal*	num_params)
{
	XtDestroyApplicationContext(XtWidgetToApplicationContext(w));
	exit(0);
}

